import json
from time import sleep

from tenacity import retry

from queries import GET_DE5_APP_BUILDS, CREATE_DE5_WORKSPACE, GET_DE5_WORKSPACE_DEPLOYMENTS
from .exceptions import handle_app_exception
from .git import config_de5_git, commit_de5_app, push_de5_app
from .constants import DEPLOY_APP_MINUTES_DELAY, RETRY_CONFIG
from .graphql import de5_request
from .users import get_de5_user_by_username
from .workspaces import upload_app_workspace_files


@retry(**RETRY_CONFIG)
def get_app_latest_commit(app):
    user = get_de5_user_by_username(app.get("owner"))
    app_name = app.get("app", {}).get("slug")

    graphql = {
        "operationName": "appBySlug",
        "variables": {"slug": app_name},
        "query": GET_DE5_APP_BUILDS
    }
    res = de5_request(graphql, user)
    res = json.loads(res.text)

    try:
        commit = res.get("data", {}).get('app', {}).get("builds", [{}])[0].get('git_revision', None)
    except:
        commit = None
    app_id = res.get("data", {}).get('app', {}).get("app_id")
    return app_id, commit


def create_app_workspace(app):
    user = get_de5_user_by_username(app.get("owner"))

    app_id, commit = get_app_latest_commit(app)

    graphql = {
        "operationName": "createOneAppDeployment",
        "variables": {
            "input": {
                "appDeployment":
                    {
                        "workspace": True,
                        "memory_limit": None,
                        "cpu_limit": None,
                        "replicas": 1,
                        "app_id": app_id,
                        "git_revision": commit
                    }

            }
        },
        "query": CREATE_DE5_WORKSPACE
    }
    return de5_request(graphql, user)


def get_is_workspace_running(app):
    app_name = app.get("app", {}).get("slug")
    user = get_de5_user_by_username(app.get("owner"))
    graphql = {
        "operationName": "appBySlug",
        "variables": {"slug": app_name},
        "query": GET_DE5_WORKSPACE_DEPLOYMENTS
    }

    res = de5_request(graphql, user)
    res = json.loads(res.text)

    app_workspaces = res.get("data", {}).get("app", {}).get("workspaces", [{}])
    
    workspace_status = None
    if len(app_workspaces) >= 1:
        workspace_status = app_workspaces[0].get("status")

    is_workspace_running = workspace_status == "running"
    return is_workspace_running


def deploy_apps(apps, folder_subname="", deploy_workspace=False, deploy_only_running=False):
    for index, app in enumerate(apps):

        if index % 4 == 0 and index != 0:
            print(f'Waiting {DEPLOY_APP_MINUTES_DELAY} minutes before deploy next pack...')
            sleep(60 * DEPLOY_APP_MINUTES_DELAY)

        owner = app.get("owner", "")
        app_name = app.get("app", {}).get("slug")
        is_running = app.get("app", {}).get("status", "")

        is_workspace_running = get_is_workspace_running(app)

        print(f"Deploy the \"{app_name}\" app | {index + 1}/{len(apps)}")

        if is_running or not deploy_only_running:
            if handle_app_exception(lambda: config_de5_git(app_name, owner, folder_subname), 'configure', app_name):
                continue
            if handle_app_exception(lambda: commit_de5_app(app_name, "Migrate to DE5 platform", folder_subname), 'commit', app_name):
                continue
            if handle_app_exception(lambda: push_de5_app(app_name, folder_subname), 'push', app_name):
                continue
        if is_workspace_running and deploy_workspace:
            if handle_app_exception(lambda: upload_app_workspace_files(app),
                                    'upload workspace files', app_name):
                continue

        print(f"\033[92mThe \"{app_name}\" app is deployed.\033[0m")
