import subprocess

from gql.transport.exceptions import TransportQueryError
from tenacity import retry

from .client import client
from .constants import RETRY_CONFIG, BASE_PATH
from .credentials import DE4_HOST
from .errors import handle_de4_api_key_error
from .file import get_from_file_raw, make_directory
from .graphql import de5_request
from queries import ATTACH_SSH, ATTACH_DE5_SSH, GET_DE4_SSH_KEYS

## uncomment line below if working on windows
# cwd = cwd.replace("\\", "/") 

DE4_KEY_FILE_NAME = f"{BASE_PATH}/migration/de4/ssh/id_rsa"
DE4_CONF_FILE_NAME = f"{BASE_PATH}/migration/de4/ssh/ssh_conf"


def get_de5_key_directory(username):
    return f"{BASE_PATH}/migration/de5/ssh/{username}"


def get_de5_key_file_name(username):
    return f"{get_de5_key_directory(username)}/id_rsa"


@retry(**RETRY_CONFIG)
def get_de4_public_key():
    return get_from_file_raw(f"{DE4_KEY_FILE_NAME}.pub")


@retry(**RETRY_CONFIG)
def generate_de4_key():
    command = ["ssh-keygen", "-t", "rsa", "-b", "4096", "-C", "dash.enterprise.migration.key", "-f", DE4_KEY_FILE_NAME, "-N", ""]
    subprocess.call(command)
    return get_de4_public_key()


def create_ssh_conf_file():
    ssh_config_text = f"Host {DE4_HOST}\n" \
                      f"\tPort 3022"
    with open(DE4_CONF_FILE_NAME, 'w') as ssh_conf_file:
        ssh_conf_file.write(ssh_config_text)

    print(f"ssh conf location in {DE4_CONF_FILE_NAME}")


@retry(**RETRY_CONFIG)
def get_de5_public_key(username):
    return get_from_file_raw(f"{get_de5_key_file_name(username)}.pub")


@retry(**RETRY_CONFIG)
def generate_de5_key(username, key_type):
    make_directory(get_de5_key_directory(username))
    if key_type == "2":
        command = ["ssh-keygen", "-t", "rsa", "-b", "4096", "-C", username, "-f", get_de5_key_file_name(username), "-N", ""]
    else:
        command = ["ssh-keygen", "-t", "ed25519", "-C", username, "-f", get_de5_key_file_name(username), "-N", ""]
    subprocess.call(command)
    return get_de5_public_key(username)


@retry(**RETRY_CONFIG)
def attach_de4_key(key):
    try:
        keys = client.execute(GET_DE4_SSH_KEYS)
        keys = keys.get("sshKeys", {}).get("keys", "")
        keys = keys.split("\n")
        keys = [key] + keys
        keys = "\n".join(keys)
        result = client.execute(ATTACH_SSH, variable_values={"keys": keys})
        print(result)
    except TransportQueryError as e:
        handle_de4_api_key_error(e)


@retry(**RETRY_CONFIG)
def attach_de5_key(key, user):
    graphql = {"operationName": "createOneSshKey",
               "variables": {
                   "input": {
                       "description": f"Key for {user.get('username')} (added via migration scripts)",
                       "ssh_key": key}
               },
               "query": ATTACH_DE5_SSH
               }
    res = de5_request(graphql, user)
    if str(res.status_code) == "503":
        raise Exception
    return res
