import json
import subprocess
import sys
from pathlib import Path
sys.path.append(Path(__file__).resolve().parent.parent.__str__())

import requests
from gql.transport.exceptions import TransportQueryError
from tenacity import retry, RetryError

from helpers.client import client
from helpers.constants import (
    RETRY_CONFIG,
    DE4_APPS_FAILED_LOAD,
    DE4_APPS_INFO_JSON_PATH,
    DE4_PORTAL_JSON_PATH,
    DE4_USERS_INFO_JSON_PATH, DE4_WORKSPACES_DATA_DIRECTORY,
)
from helpers.errors import handle_de4_api_key_error
from helpers.file import save_to_file
from helpers.credentials import DE4_HOST, DE4_ADMIN_USERNAME, DE4_ADMIN_TOKEN, DE4_SSH_PORT, DE4_SSH_PRIVATE_KEY_PATH, \
    DE4_SSH_USER
from helpers.get_app_names import get_app_names
from queries import GET_APP_INFO, GET_PORTAL_INFO


@retry(**RETRY_CONFIG)
def get_app_info(app_name):
    try:
        result = client.execute(GET_APP_INFO, variable_values={"name": app_name})
        apps = result.get('apps', {}).get('apps', [])
        return apps[0] if len(apps) > 0 else {}
    except TransportQueryError as e:
        handle_de4_api_key_error(e)


def get_apps():
    apps = []
    failed_apps = []

    names = get_app_names()
    total_names = len(names)
    for index, name in enumerate(names):
        try:
            app = get_app_info(name)
            if app.get('name') is None:
                continue
            apps.append(app)
            print(f"[App processing]: Processed {index + 1} of {total_names} apps: \"{name}\".")
        except RetryError as e:
            failed_apps.append(name)
            print(f"\033[91mFailed to get the \"{name}\" app info. This app was skipped. See the reason below.\033[0m")
            print(e.last_attempt.exception())

    save_to_file(DE4_APPS_FAILED_LOAD, failed_apps)


    failed_apps_count = len(failed_apps)

    if failed_apps_count > 0:
        print(f"Failed apps count: {failed_apps_count}")

    return apps


@retry(**RETRY_CONFIG)
def get_portal_info():
    try:
        result = client.execute(GET_PORTAL_INFO, variable_values={"name": "default"})
        return result["portals"]["portals"][0]
    except TransportQueryError as e:
        handle_de4_api_key_error(e)


@retry(**RETRY_CONFIG)
def get_users():
    users = []
    url = f"https://{DE4_ADMIN_USERNAME}:{DE4_ADMIN_TOKEN}@{DE4_HOST}/Auth/v2/users?page_size=50"
    headers = {
        "Content-Type": "application/json",
        "plotly-client-platform": "dash-auth"
    }

    while url:
        result = requests.get(url, headers=headers).json()
        users += list(map(
            lambda user: {
                "email": user["email"],
                "admin": user["is_admin"],
                "licensed": user["is_dash_creator"],
                "username": user["username"],
            },
            result["results"]
        ))

        if result['next'] and "172.17.0.1:8008" in result['next']:
            url = result['next'].replace("172.17.0.1:8008", f'{DE4_ADMIN_USERNAME}:{DE4_ADMIN_TOKEN}@{DE4_HOST}')
        else:
            url = None
        
        print(f"[Get users processing]: Processed {len(users)} users.")

    return users


def get_local_app_names():
    f = open(DE4_APPS_INFO_JSON_PATH, "r")
    apps = json.loads(f.read())
    return list(map(lambda item: item["name"], apps))


@retry(**RETRY_CONFIG)
def get_workspaces_data():
    app_names = get_local_app_names()

    for name in app_names:
        print(f"Downloading workspace files for the {name} app")
        rsync_command = [
            'rsync',
            '-avz',
            '--delete',
            '-e',
            f'ssh -p {DE4_SSH_PORT} -i {DE4_SSH_PRIVATE_KEY_PATH}',
            f'--rsync-path=sudo rsync',
            f'{DE4_SSH_USER}@{DE4_HOST}:/plotly/dash/workspaces/{name}/',
            f'{DE4_WORKSPACES_DATA_DIRECTORY}/{name}'
        ]
        subprocess.call(rsync_command, stdout=subprocess.DEVNULL)


def with_logs(name, method):
    print(f"[{name}]: Started.")
    method()
    print(f"[{name}]: Finished.")


if __name__ == "__main__":
    with_logs("App processing", lambda: save_to_file(DE4_APPS_INFO_JSON_PATH, get_apps()))
    with_logs("Get portal info", lambda: save_to_file(DE4_PORTAL_JSON_PATH, get_portal_info()))
    with_logs("Get users", lambda: save_to_file(DE4_USERS_INFO_JSON_PATH, get_users()))
    with_logs("Get workspaces data", lambda: get_workspaces_data())

