import sys
from pathlib import Path
sys.path.append(Path(__file__).resolve().parent.parent.__str__())

import json
import random
import string
import re
import requests
from markdownify import markdownify

from helpers.constants import DE4_APPS_INFO_JSON_PATH, DE5_APPS_INFO_PATH
from helpers.constants import DE5_USERS_PATH
from helpers.credentials import DE4_HOST
from helpers.file import save_to_file, delete_file
from helpers.image import resize_image_to_match_max_size, convert_image_to_base64
from helpers.users import get_de4_users

PASSWORD_LENGTH = 10

DOMAIN_PATTERN = r"^[a-z0-9][a-z0-9-]{1,61}[a-z0-9]\.[a-z]{2,}$"


def get_apps():
    f = open(DE4_APPS_INFO_JSON_PATH, "r")
    return json.loads(f.read())


def safety(value, default):
    return value if value is not None else default


MAX_IMAGE_SIZE = 100*1024


def prepare_base_64_thumbnail(thumbnail):
    if thumbnail is not None:
        url = f"https://{DE4_HOST}/{thumbnail}"
        name = url.split("/")[-1]
        content = requests.get(f"https://{DE4_HOST}/{thumbnail}").content
        save_to_file(name, content, is_json=False, binary=True)
        resize_image_to_match_max_size(name, MAX_IMAGE_SIZE)
        thumbnail = convert_image_to_base64(name)
        delete_file(name)

    return thumbnail


def convert_to_de5_format(app, is_idp):
    print(f"The \"{app.get('name')}\" app is processing.")
    metadata = safety(app.get("metadata"), {})
    collaborators = safety(app.get("collaborators"), {})
    variables = safety(app.get("environmentVariables"), [])
    services = safety(app.get("linkedServices"), [])
    status = safety(app.get("status"), {}).get("running", False)
    tags = safety(metadata.get("tags"), "")
    view_access = metadata.get("permissionLevel")

    app_users = collaborators.get('users', [])

    if is_idp:
        new_username = check_username_format(app['owner']['username'])
        if new_username:
            print(
                "App's owner username for app {} is changed from: {} to: {}".format(
                    app["name"],
                    app['owner']['username'],
                    new_username
                )
            )
            app['owner']['username'] = new_username
        app_users = []
        for app_viewer_user in collaborators.get('users', []):
            new_app_user_username = check_username_format(app_viewer_user)
            if new_app_user_username:
                print(
                    "App viewer's username for app {} is changed from: {} to: {}".format(
                        app["name"],
                        app_viewer_user,
                        new_username
                    )
                )
            app_users.append(
                new_app_user_username or app_viewer_user
            )

    if view_access == "specific":
        view_access = "restricted"

    thumbnail = safety(app.get("thumbnailUrl"), None)
    thumbnail = prepare_base_64_thumbnail(thumbnail)

    description = metadata.get("description")
    if description and description is not None:
        description = markdownify(description)

    default_variables = [{"key": "LC_ALL", "value": "C.UTF-8"}, {"key": "LANG", "value": "C.UTF-8", }]
    environment_variables = [{"key": x["name"], "value": x["value"]} for x in variables if not x["readonly"]]
    environment_variables += default_variables

    return {
        "app": {
            "slug": app.get("name"),
            "title": metadata.get("title"),
            "description": description,
            "tags": list(map(
                lambda tag: {"slug": tag, "color": "rgb(30,30,30)"},
                tags.split(",")
            )) if len(tags) > 0 else [],
            "view_access": view_access,
            "viewers": app_users,
            "group_viewers": collaborators.get('teams', []),
            "thumbnail": thumbnail,
            "environment_variables": environment_variables,
            "visible_on_portal": metadata.get("showInPortal"),
            "status": status,
        },
        "services": list(map(
            lambda x: {"name": x["name"], "type": x["serviceType"]},
            services
        )),
        "owner": app.get("owner", {}).get("username")
    }


def check_username_format(username):
    divide_index = username.rfind('_')

    # check last part is domain
    if divide_index > 0:
        maybe_domain = username[divide_index + 1:]

        # convert domain to lower
        if maybe_domain:
            if re.fullmatch(DOMAIN_PATTERN, maybe_domain.lower()):
                new_username = '{}@{}'.format(
                    username[:divide_index].lower(),
                    maybe_domain.lower()
                )
                return new_username


def get_random_string(length):
    letters = string.ascii_letters + string.digits + string.punctuation
    return "".join(random.choice(letters) for _ in range(length))


def get_random_password():
    return get_random_string(PASSWORD_LENGTH)


def convert_users_for_de5(users, is_idp):
    if is_idp:
        return convert_users_for_de5_idp(users)
    return [{**user, "password": "Password123!"} for user in users]

def convert_users_for_de5_idp(de4_users):
    d5_users = []
    for user in de4_users:
        print('user in progress: {}'.format(user['username']))

        new_username = check_username_format(user["username"])
        if new_username:
            if not user["email"]:
                # update email if it doesn't exist
                user["email"] = new_username
                user["username"] = new_username
            elif user["email"] == new_username:
                # set username as the email if its equal
                user["username"] = new_username
        d5_users.append({**user, "password": "Password123!"})
    return d5_users



def main():
    answer = input("Do you have users from the external idp? (yes/no): ")
    is_idp_migration = answer == "yes"
    print(is_idp_migration)
    de4_users = get_de4_users()
    users = convert_users_for_de5(de4_users, is_idp_migration)
    save_to_file(DE5_USERS_PATH, users)
    apps = get_apps()
    converted_apps = []

    for app in apps:
        converted = convert_to_de5_format(app, is_idp_migration)
        if converted:
            converted_apps.append(converted)

    save_to_file(DE5_APPS_INFO_PATH, converted_apps)


if __name__ == "__main__":
    main()
