import sys
from pathlib import Path

sys.path.append(Path(__file__).resolve().parent.parent.__str__())

from gql.transport.exceptions import TransportQueryError
from tenacity import retry

from helpers.client import client
from helpers.constants import (
    DE5_APPS_INFO_PATH,
    DE5_APPS_PATH,
    DE4_APPS_PATH,
    DE5_APPS_SLICING,
    RETRY_CONFIG,
)
from helpers.credentials import DE5_HOST, DE5_INTERNAL_PYPI_SERVICE
from helpers.errors import handle_de4_api_key_error
from helpers.file import (
    get_from_file,
    copy_directory,
    is_file_exists,
    get_from_file_raw,
    remove_file_if_exists,
    save_to_file,
    save_as_toml_file,
    save_as_yaml_file,
    load_from_toml_file,
    delete_file,
)
from queries import GET_APP_DEPENDENCY
from urllib.parse import urlparse


def get_apps():
    return get_from_file(DE5_APPS_INFO_PATH)


def get_app_directory(base_directory, app_name):
    return f"{base_directory}/{app_name}"


def get_main_app_directory(app_name):
    return get_app_directory(DE5_APPS_PATH, f"{app_name}-old-python")


def convert_apt(app_name):
    app_root = get_main_app_directory(app_name)

    def get_dependencies_from_file_name(file_name):
        file_path = get_app_directory(app_root, file_name)
        if is_file_exists(file_path):
            res = get_from_file_raw(file_path)
            return res.split("\n")
        return []

    apt_file_names = ["apt-packages", "dpkg-packages", "apt-repositories"]

    dependencies = []

    for file in apt_file_names:
        dependencies += get_dependencies_from_file_name(file)
        remove_file_if_exists(get_app_directory(app_root, file))

    dependencies = "\n".join(dependencies)
    save_to_file(get_app_directory(app_root, "Aptfile"), dependencies, is_json=False)


def get_correct_version(version):
    result = "3.6.15"
    if "3.6" in version:
        result = "3.6.*"

    if "3.7" in version:
        result = "3.7.*"

    if "3.8" in version:
        result = "3.8.*"

    if "3.9" in version:
        result = "3.9.*"

    return result


def build_project_toml(app_name):
    app_root = get_main_app_directory(app_name)
    result = {}

    app_json_path = get_app_directory(app_root, "app.json")

    if is_file_exists(app_json_path):
        app_json = get_from_file(app_json_path)
        result["scripts"] = {}

        predeploy = app_json.get("scripts", {}).get("predeploy", "")
        result["scripts"]["predeploy"] = predeploy
        result["scripts"]["postdeploy"] = app_json.get("scripts", {}).get(
            "postdeploy", ""
        )

    runtime_path = get_app_directory(app_root, "runtime.txt")

    version = "3.6"

    if is_file_exists(runtime_path):
        runtime = get_from_file_raw(runtime_path)
        version = runtime.replace("python-", "").replace("\n", "")

    version = get_correct_version(version)

    result["build"] = {"env": [{}]}
    result["build"]["env"][0]["name"] = "BP_CPYTHON_VERSION"
    result["build"]["env"][0]["value"] = version

    toml_path = get_app_directory(app_root, "project.toml")
    save_as_toml_file(result, toml_path)


@retry(**RETRY_CONFIG)
def get_installed_requirements(app_name):
    try:
        result = client.execute(GET_APP_DEPENDENCY, variable_values={"name": app_name})
        dependencies = (
            result.get("apps", {})
            .get("apps", [{}])[0]
            .get("analytics", {})
            .get("dependencies", {})
            .get("python", "")
        )
        dependencies = dependencies.split("\n\n")
        return dependencies
    except TransportQueryError as e:
        handle_de4_api_key_error(e)


def make_a_copy_with_old_pip(app_name):
    app_root = get_app_directory(DE5_APPS_PATH, f"{app_name}-old-pip")
    copied_directory_path = get_app_directory(DE5_APPS_PATH, app_name)
    copy_directory(copied_directory_path, app_root)
    requirements_file_path = get_app_directory(app_root, "requirements.txt")
    requirements_base_file_path = get_app_directory(app_root, "requirements-base.txt")

    if not is_file_exists(requirements_file_path):
        return

    requirements = get_from_file_raw(requirements_file_path)

    save_to_file(requirements_base_file_path, requirements, is_json=False)
    save_to_file(requirements_file_path, "pip==20.2.4", is_json=False)

    toml_path = get_app_directory(app_root, "project.toml")
    toml = load_from_toml_file(toml_path)

    predeploy_file_name = "predeploy.sh"
    predeploy_path = get_app_directory(app_root, predeploy_file_name)

    scripts = toml.get("scripts", {})
    predeploy = scripts.get("predeploy", "")

    if len(predeploy) > 0:
        predeploy += " && "
    predeploy += predeploy_file_name

    scripts["predeploy"] = predeploy
    toml["scripts"] = scripts

    save_as_toml_file(toml, toml_path)

    predeploy_file_content = 'echo "######### INSTALLING PACKAGES ######### "\npip install -r requirements-base.txt'
    save_to_file(predeploy_path, predeploy_file_content, is_json=False)


def update_requirements(app_name):
    installed_requirements = get_installed_requirements(app_name)
    app_root = get_main_app_directory(app_name)
    requirements_file_path = get_app_directory(app_root, "requirements.txt")

    requirements = installed_requirements

    updated_requirements = [
        f"--extra-index-url https://{DE5_HOST}/packages/",
    ]

    if DE5_INTERNAL_PYPI_SERVICE:
        updated_requirements.append(f"--extra-index-url {DE5_INTERNAL_PYPI_SERVICE}")
        updated_requirements.append(
            f"--trusted-host {urlparse(DE5_INTERNAL_PYPI_SERVICE).hostname}"
        )

    for r in requirements:
        updated_r = r

        dependency_name = get_package_name_from_dependency(updated_r)

        if dependency_name == "dataclasses":
            continue

        if dependency_name == "pandas":
            updated_r = "pandas"

        if " @" in updated_r:
            updated_r = updated_r.split("file:///tmp/build/")[-1]

        updated_requirements.append(updated_r)

    save_to_file(requirements_file_path, "\n".join(updated_requirements), is_json=False)


def get_package_name_from_dependency(dependency):
    if "==" in dependency:
        return dependency.split("==")[0]

    if ">=" in dependency:
        return dependency.split(">=")[0]

    if "~=" in dependency:
        return dependency.split("~=")[0]

    return dependency


def build_conda_yaml(app_name):
    app_root = get_main_app_directory(app_name)

    conda_requirements_path = get_app_directory(app_root, "conda-requirements.txt")

    if not is_file_exists(conda_requirements_path):
        return

    conda_requirements = get_from_file_raw(conda_requirements_path)
    result = conda_requirements.split("\n")

    requirements_path = get_app_directory(app_root, "requirements.txt")

    if is_file_exists(requirements_path):
        requirements = get_from_file_raw(requirements_path)
        requirements = requirements.split("\n")
        result.append({"pip": requirements})

    save_as_yaml_file(
        {
            "name": "my-conda-env",
            "channels": ["conda-forge", "defaults"],
            "dependencies": result,
        },
        get_app_directory(app_root, "environment.yml"),
    )

    delete_file(get_app_directory(app_root, "requirements.txt"))
    delete_file(conda_requirements_path)


def make_a_copy_with_default_version(app_name):
    copied_directory_path = get_app_directory(DE5_APPS_PATH, f"{app_name}")
    copy_directory(get_main_app_directory(app_name), copied_directory_path)
    toml_path = get_app_directory(copied_directory_path, "project.toml")
    toml = load_from_toml_file(toml_path)
    toml.pop("build")
    save_as_toml_file(toml, toml_path)


def process_app(app):
    app_name = app.get("app", {}).get("slug")
    copy_directory(
        get_app_directory(DE4_APPS_PATH, app_name), get_main_app_directory(app_name)
    )
    convert_apt(app_name)
    update_requirements(app_name)
    build_project_toml(app_name)
    build_conda_yaml(app_name)
    make_a_copy_with_default_version(app_name)
    make_a_copy_with_old_pip(app_name)
    print(f'The "{app_name}" app is processed.')


if __name__ == "__main__":
    apps = get_apps()
    for app in apps[DE5_APPS_SLICING]:
        try:
            process_app(app)
        except Exception as e:
            app_name = app.get("app", {}).get("slug")
            print(e)
            print(f"\033[91mThe \"{app_name} app wasn't  converted. Ensure that app was cloned successfully.\"\033[0m")
