import sys
from pathlib import Path
sys.path.append(Path(__file__).resolve().parent.parent.__str__())

from keycloak import KeycloakAdmin
from tenacity import retry

from helpers.constants import DE5_APPS_INFO_PATH, RETRY_CONFIG, DE5_FAILED_USERS_PATH
from helpers.credentials import DE5_KEYCLOAK_ADMIN_PASSWORD, DE5_HOST, DE5_KEYCLOAK_ADMIN_USERNAME
from helpers.file import get_from_file, save_to_file
from helpers.users import get_de5_users

admin_pass = DE5_KEYCLOAK_ADMIN_PASSWORD
keycloak_url = f"https://auth-{DE5_HOST}/auth/"


def get_apps():
    return get_from_file(DE5_APPS_INFO_PATH)


keycloak_admin = KeycloakAdmin(server_url=keycloak_url,
                               username=DE5_KEYCLOAK_ADMIN_USERNAME,
                               password=admin_pass,
                               realm_name='dash',
                               user_realm_name='master',
                               verify=True)


def attach_role(role_name, user_id):
    dash_client_id = keycloak_admin.get_client_id("dash")
    va_role_id = keycloak_admin.get_client_role_id(
        client_id=dash_client_id,
        role_name=role_name
    )
    va_role = {
        "id": va_role_id,
        "name": role_name,
        "composite": False,
        "clientRole": True,
        "containerId": dash_client_id
    }
    keycloak_admin.assign_client_role(
        client_id=dash_client_id,
        user_id=user_id,
        roles=va_role
    )


@retry(**RETRY_CONFIG)
def create_user(user):
    try:
        new_user = keycloak_admin.create_user({
            "email": user["email"],
            "username": user["username"],
            "enabled": True,
            "credentials": [{"value": user["password"], "type": "password", "temporary": False}]
        }, exist_ok=False)

        if user["licensed"]:
            attach_role('licensed_user', new_user)

        if user["admin"]:
            attach_role('admin', new_user)

        return True
    except Exception as e:
        print(f"\033[91mCan't create user with this username: {user['username']}."
              f" You can see the reason under this message.\033[0m")
        print(e)
        return False


def get_groups():
    apps = get_apps()
    groups = []
    for app in apps:
        groups.extend(app.get("app", {}).get("group_viewers", []))

    return list(set(groups))


@retry(**RETRY_CONFIG)
def create_one_group(name):
    try:
        keycloak_admin.create_group({"name": name})
        print(f"The \"{name}\" is created")
        return True
    except Exception as exc:
        print(f"\033[91mCan't create group with this name: {name}. You can see the reason under this message.\033[0m")
        print(exc)
        return False


def create_groups():
    groups = get_groups()

    groups_created_count = 0
    for group in groups:
        created = create_one_group(group)
        if created:
            groups_created_count += 1
    print("{}/{} groups created".format(
        groups_created_count,
        len(groups),
    ))


if __name__ == "__main__":
    users = get_de5_users()
    failed_users = []
    users_created_count = 0
    for user in users:
        is_created = create_user(user)

        if is_created:
            users_created_count += 1
        else:
            failed_users.append(user)

    save_to_file(DE5_FAILED_USERS_PATH, failed_users)
    print("{}/{} users failed, {}/{} users created".format(
        len(failed_users),
        len(users),
        users_created_count,
        len(users)
    ))
    create_groups()
