import sys
from pathlib import Path
import json

from tenacity import retry, RetryError
from time import sleep

sys.path.append(Path(__file__).resolve().parent.parent.__str__())

from helpers.deployment_queue import workspaces_queue
from helpers.constants import DE5_APPS_INFO_PATH, DE5_APPS_SLICING, RETRY_CONFIG
from helpers.file import get_from_file
from helpers.graphql import de5_request
from helpers.users import get_de5_user_by_username
from queries import UPDATE_APP, CREATE_SERVICE


def get_apps():
    return get_from_file(DE5_APPS_INFO_PATH)


def safety(value, default):
    return value if value is not None else default


def create_raw_app(app, user):
    app.pop("environment_variables", None)
    app.pop("status", None)
    graphql = {"operationName": "createApp",
               "variables": {
                   "input": {
                       "app": app
                   }
               },
               "query": "mutation createApp($input: CreateOneAppInput!) {\n  createOneApp(input: $input) {\n    app_id\n    title\n    __typename\n  }\n}\n"
               }

    res = de5_request(graphql, user)
    errors = safety(json.loads(res.text).get('errors'), [])
    if len(errors) > 0:
        errors_messages = list(map(lambda item: item.get("message", ""), errors))
        formatted_errors = '\n'.join(errors_messages)
        print(f"\033[91mCan’t create apps for the following reasons:\n{formatted_errors}\033[0m")

        return None
    return safety(json.loads(res.text).get('data'), {}).get('createOneApp', {}).get('app_id')


@retry(**RETRY_CONFIG)
def attach_environment_variables(app_id, variables, user):
    if len(variables) == 0:
        print("Environment variables is empty, skipped.")
        return

    graphql = {"operationName": "updateApp",
               "variables": {
                   "input": {
                       "id": app_id,
                       "app": {
                           "environment_variables": variables
                       }
                   }
               },
               "query": UPDATE_APP
               }
    res = de5_request(graphql, user)

    print(res.text)

    return res


@retry(**RETRY_CONFIG)
def attach_viewers(app_id, viewers, user):
    if len(viewers) == 0:
        print("Viewers is empty, skipped.")
        return

    graphql = {"operationName": "updateApp",
               "variables": {
                   "input": {
                       "id": app_id,
                       "app": {
                           "viewers": viewers
                       }
                   }
               },
               "query": UPDATE_APP
               }
    res = de5_request(graphql, user)
    return res


@retry(**RETRY_CONFIG)
def attach_services(app_id, services, user):
    if len(services) == 0:
        print("Services is empty, skipped.")
        return

    for service in services:
        create_service(app_id, service, user)


@retry(**RETRY_CONFIG)
def create_service(app_id, service, user):
    graphql = {"operationName": "createService",
               "variables": {
                   "input": {
                       "app_id": app_id,
                       "name": service["name"],
                       "type": service["type"],
                       "config": {
                           "cpu_limit": 1000,
                           "memory_limit": 512,
                           "storage_limit": 1024,
                           "replicas_limit": 1
                       }
                   }
               },
               "query": CREATE_SERVICE
               }
    res = de5_request(graphql, user)
    return res


def create_app(app):
    user = get_de5_user_by_username(app.get("owner"))

    de5_app = app["app"]
    services = app["services"]

    attempts = 1
    app_id = create_raw_app({**de5_app}, user)

    while app_id is None and attempts < 2:
        sleep_time = 30 * (2 ** (attempts - 1))
        print(f"Failed to create app. Will be retried in {sleep_time} seconds")
        sleep(sleep_time)
        attempts += 1
        app_id = create_raw_app({**de5_app}, user)

    if app_id is None:
        return False

    attach_environment_variables(app_id, de5_app["environment_variables"], user)
    attach_viewers(app_id, de5_app["viewers"], user)
    attach_services(app_id, services, user)
    return True


if __name__ == "__main__":
    apps = get_apps()
    for app in apps[DE5_APPS_SLICING]:
        app_name = app.get("app", {}).get("slug")
        print(f"\033[92mStart the \"{app_name}\" app creation.\033[0m")

        try:
            created = create_app(app)
        except RetryError as e:
            print(e.last_attempt.exception())
            print(f"\033[91mThe \"{app_name}\" app creation is failed.\033[0m")
            continue
        if created:
            workspaces_queue.enqueue(app)
            print("Workspace creating pushed to queue")
            print(f"\033[92mThe \"{app_name}\" app creation process is finished.\033[0m")
        else:
            print(f"\033[91mThe \"{app_name}\" app creation is failed.\033[0m")

